/******************************************************************************
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Id: check_printer.c,v 1.31 2004/12/25 23:17:44 opensides Exp $
*****************************************************************************/

const char *progname = "check_printer";
const char *revision = "$Revision: 0.12 $";
const char *copyright = "2000-2005";
const char *email = "csc@fes-aes.de";

#include "common.h"
#include "popen.h"
#include "utils.h"
#include "netutils.h"

#define DEFAULT_COMMUNITY "public"


const char *option_summary = "-H host [-C community]\n";

#define PTR_CONSOLE_DISPLAY_BUFFER_TEXT	".1.3.6.1.2.1.43.16"
#define	PTR_ALERT_TABLE			".1.3.6.1.2.1.43.18"
#define	PTR_ALERT_INDEX			".1.3.6.1.2.1.43.18.1.1.1.1"
#define	PTR_ALERT_SEVERITY_LEVEL	".1.3.6.1.2.1.43.18.1.1.2.1"
#define	PTR_ALERT_TRAINING_LEVEL	".1.3.6.1.2.1.43.18.1.1.3.1"
#define	PTR_ALERT_GROUP			".1.3.6.1.2.1.43.18.1.1.4.1"
#define	PTR_ALERT_GROUP_INDEX		".1.3.6.1.2.1.43.18.1.1.5.1"
#define	PTR_ALERT_LOCATION		".1.3.6.1.2.1.43.18.1.1.6.1"
#define	PTR_ALERT_CODE			".1.3.6.1.2.1.43.18.1.1.7.1"
#define	PTR_ALERT_DESCRIPTION		".1.3.6.1.2.1.43.18.1.1.8.1"
#define	PTR_ALERT_TIME			".1.3.6.1.2.1.43.18.1.1.9.1"

#define LEVEL_UNTRAINED			3
#define LEVEL_TRAINED			4
#define LEVEL_FIELDSERVICE		5


int process_arguments (int, char **);
int validate_arguments (void);
void print_help (void);
void print_usage (void);
char* convert (char*);

char *community = NULL;
char *address = NULL;
int training_level = LEVEL_TRAINED;

int main (int argc, char **argv) {

	char command_line[1024];
	int result = STATE_UNKNOWN;
	int line    = 0;
	int index   = 0;
	int entries = 0;
	int id[65535];
	char input_buffer[MAX_INPUT_BUFFER];
	char *errmsg;
	char *temp_buffer;
	unsigned short int code_mask[2048]; /* specify mask for binary change events which be should considered critical */
				/* other(1) */
			        /* unknown(2) */
			        /* coverOpen(3) */
			        /* coverClosed(4) */
			        /* interlockOpen(5) */
			        /* interlockClosed(6) */
			        /* configurationChange(7) */
        code_mask[8] = TRUE;	/* jam(8) */
        code_mask[9] = TRUE;	/* subunitMissing(9) 			-- Not in RFC 1759 */
				/* subunitLifeAlmostOver(10)		-- Not in RFC 1759 */
        code_mask[11] = TRUE;	/* subunitLifeOver(11)			-- Not in RFC 1759 */
				/* subunitAlmostEmpty(12)		-- Not in RFC 1759 */
        code_mask[13] = TRUE;	/* subunitEmpty(13)			-- Not in RFC 1759 */
    				/* subunitAlmostFull(14)		-- Not in RFC 1759 */
				/* subunitFull(15)			-- Not in RFC 1759 */
				/* subunitNearLimit(16)			-- Not in RFC 1759 */
			        /* subunitAtLimit(17)			-- Not in RFC 1759 */
			        /* subunitOpened(18)			-- Not in RFC 1759 */
			        /* subunitClosed(19)			-- Not in RFC 1759 */
			        /* subunitTurnedOn(20)			-- Not in RFC 1759 */
        code_mask[21] = TRUE;	/* subunitTurnedOff(21)			-- Not in RFC 1759 */
        code_mask[22] = TRUE;	/* subunitOffline(22)			-- Not in RFC 1759 */
			        /* subunitPowerSaver(23)		-- Not in RFC 1759 */
			        /* subunitWarmingUp(24)			-- Not in RFC 1759 */
			        /* subunitAdded(25)			-- Not in RFC 1759 */
        code_mask[26] = TRUE;	/* subunitRemoved(26)			-- Not in RFC 1759 */
			        /* subunitResourceAdded(27)		-- Not in RFC 1759 */
        code_mask[28] = TRUE;	/* subunitResourceRemoved(28)		-- Not in RFC 1759 */
				/* subunitRecoverableFailure(29)	-- Not in RFC 1759 */
        code_mask[30] = TRUE;	/* subunitUnrecoverableFailure(30)	-- Not in RFC 1759 */
          			/* subunitRecoverableStorageError(31)	-- Not in RFC 1759 */
        code_mask[32] = TRUE;	/* subunitUnrecoverableStorageError(32)	-- Not in RFC 1759 */
	code_mask[33] = TRUE;	/* subunitMotorFailure(33)		-- Not in RFC 1759 */
	code_mask[34] = TRUE;	/* subunitMemoryExhausted(34)		-- Not in RFC 1759 */
				/* subunitUnderTemperature(35)		-- Not in RFC 1759 */
				/* subunitOverTemperature(36)		-- Not in RFC 1759 */
				/* subunitTimingFailure(37)		-- Not in RFC 1759 */
				/* subunitThermistorFailure(38)		-- Not in RFC 1759 */

				/* doorOpen(501)			-- DEPRECATED - Use coverOpened(3) */
				/* doorClosed(502)  			-- DEPRECATED - Use coverClosed(4) */
				/* powerUp(503) */
				/* powerDown(504) */
				/* printerNMSReset(505)			-- Not in RFC 1759 */
				/* printerManualReset(506)		-- Not in RFC 1759 */
				/* printerReadyToPrint(507)		-- Not in RFC 1759 */

	code_mask[801] = TRUE;	/* inputMediaTrayMissing(801) */
				/* inputMediaSizeChange(802) */
				/* inputMediaWeightChange(803) */
				/* inputMediaTypeChange(804) */
				/* inputMediaColorChange(805) */
				/* inputMediaFormPartsChange(806) */
				/* inputMediaSupplyLow(807) */
	code_mask[808] = TRUE;	/* inputMediaSupplyEmpty(808) */
				/* inputMediaChangeRequest(809)		-- Not in RFC 1759 */
				/* inputManualInputRequest(810)		-- Not in RFC 1759 */
				/* inputTrayPositionFailure(811)	-- Not in RFC 1759 */
				/* inputCannotFeedSizeSelected(813)	-- Not in RFC 1759 */
				/* inputTrayElevationFailure(812)	-- Not in RFC 1759 */

	code_mask[901] = TRUE;	/* outputMediaTrayMissing(901) */
				/* outputMediaTrayAlmostFull(902) */
	code_mask[903] = TRUE;	/* outputMediaTrayFull(903) */
				/* outputMailboxSelectFailure(904)	-- Not in RFC 1759 */

				/* markerFuserUnderTemperature(1001) */
				/* markerFuserOverTemperature(1002) */
				/* markerFuserTimingFailure(1003)	-- Not in RFC 1759 */
				/* markerFuserThermistorFailure(1004)	-- Not in RFC 1759 */
				/* markerAdjustingPrintQuality(1005)	-- Not in RFC 1759 */
	code_mask[1101] = TRUE;	/* markerTonerEmpty(1101) */
	code_mask[1102] = TRUE;	/* markerInkEmpty(1102) */
				/* markerPrintRibbonEmpty(1103) */
				/* markerTonerAlmostEmpty(1104) */
				/* markerInkAlmostEmpty(1105) */
				/* markerPrintRibbonAlmostEmpty(1106) */
				/* markerWasteTonerReceptacleAlmostFull(1107) */
				/* markerWasteInkReceptacleAlmostFull(1108) */
				/* markerWasteTonerReceptacleFull(1109) */
				/* markerWasteInkReceptacleFull(1110) */
				/* markerOpcLifeAlmostOver(1111) */
	code_mask[1112] = TRUE;	/* markerOpcLifeOver(1112) */
				/* markerDeveloperAlmostEmpty(1113) */
	code_mask[1114] = TRUE;	/* markerDeveloperEmpty(1114) */
	code_mask[1115] = TRUE;	/* markerTonerCartridgeMissing(1115)	-- Not in RFC 1759 */

	code_mask[1301] = TRUE;	/* mediaPathMediaTrayMissing(1301) */
				/* mediaPathMediaTrayAlmostFull(1302) */
	code_mask[1303] = TRUE;	/* mediaPathMediaTrayFull(1303) */
				/* mediaPathCannotDuplexMediaSelected(1304) -- Not in RFC 1759 */
				/* interpreterMemoryIncrease(1501) */
				/* interpreterMemoryDecrease(1502) */
				/* interpreterCartridgeAdded(1503) */
				/* interpreterCartridgeDeleted(1504) */
				/* interpreterResourceAdded(1505) */
				/* interpreterResourceDeleted(1506) */
				/* interpreterResourceUnavailable(1507) */
				/* interpreterComplexPageEncountered(1509) -- Not in RFC 1759 */
				/* alertRemovalOfBinaryChangeEntry(1801) -- Not in RFC 1759 */

	typedef struct prtTableEntry {
	        int severity;
		int training;
		/* int group; */
		/* int group_index; */
		/* int location; */
		int code;
		char *descr;
		/* int time; */
	} aPrtTableEntry;
	
	aPrtTableEntry entry[256]; /* should be enough */

	errmsg = malloc(MAX_INPUT_BUFFER);

	setlocale (LC_ALL, "");
	bindtextdomain (PACKAGE, LOCALEDIR);
	textdomain (PACKAGE);

	if (process_arguments (argc, argv) == ERROR)
		usage4 (_("Could not parse arguments"));

	/* get the command to run */
	#if NETSNMP
	sprintf (command_line, "%s -On -Oe -Oq -Oa -v1 -c %s %s %s",
	#else
	sprintf (command_line, "%s -On -Oe -Oq -v1 -c %s %s %s",
	#endif
		PATH_TO_SNMPWALK, community, address, PTR_ALERT_TABLE);

	#if DEBUG
	printf("command   : %s\n",command_line);
	#endif

	/* run the command */
	child_process = spopen (command_line);
	
	if (child_process == NULL) {
		printf (_("Could not open pipe: %s\n"), command_line);
		return STATE_UNKNOWN;
	}

	child_stderr = fdopen (child_stderr_array[fileno (child_process)], "r");
	if (child_stderr == NULL) {
		printf (_("Could not open stderr for %s\n"), command_line);
	}

	while (fgets (input_buffer, MAX_INPUT_BUFFER - 1, child_process)) {

		/* strip the newline character from the end of the input */
		if (input_buffer[strlen (input_buffer) - 1] == '\n')
			input_buffer[strlen (input_buffer) - 1] = 0;

		#if DEBUG
		printf("cmd output: %s\n",input_buffer);
		#endif

		temp_buffer = strstr(input_buffer," ") + 1;
		sscanf(input_buffer,".1.3.6.1.2.1.43.18.1.1.%*d.1.%d",&index); /* prtAlertIndex */

		if (strstr(input_buffer,PTR_ALERT_SEVERITY_LEVEL)) {
			id[index] = entries++;
		        entry[id[index]].severity = atoi(temp_buffer);
		}

		if (strstr(input_buffer,PTR_ALERT_TRAINING_LEVEL))
			entry[id[index]].training = atoi(temp_buffer);

		if (strstr(input_buffer,PTR_ALERT_CODE))
			entry[id[index]].code = atoi(temp_buffer);

		if (strstr(input_buffer,PTR_ALERT_DESCRIPTION)) {
			entry[id[index]].descr = strcpy((char*)malloc(MAX_INPUT_BUFFER),convert(temp_buffer));
		}
		
		if (!(strstr(input_buffer,PTR_ALERT_TABLE))) { /* AlertDescription is hex and more than one line */
			entry[id[index]].descr = strcat(entry[id[index]].descr,convert(input_buffer));
		}
		
	}

	for (line=0;line<=entries-1;line++) {

		#if DEBUG
		printf("      ==> : line=%d: severity=%d training=%d => \"%s\"\n",line,entry[line].severity,entry[line].training,entry[line].descr);
		#endif

		/* only care for states which require intervention of service personal */
		if (training_level <= entry[line].training && entry[line].training <= LEVEL_FIELDSERVICE) {

			/* use the last (latest?) AlarmTableEntry with the highest priority */
			switch (entry[line].severity) {
				
	    			case 1: /* other */

					result = max_state(result,STATE_OK);
					if (result == STATE_OK)
						if (entry[line].descr)
				    			errmsg = entry[line].descr;
					break;

	    			case 3: /* critical */

					result = STATE_CRITICAL;
					if (entry[line].descr)
						errmsg = entry[line].descr;
					break;

				case 4: /* warning */

					result = max_state(result,STATE_WARNING);
					if (result == STATE_WARNING)
						if (entry[line].descr)
							errmsg = entry[line].descr;
					break;

	    			case 5: /* warningBinaryChangeEvent -- New, not in RFC 1759 */

					result = max_state(result,STATE_WARNING);
					if (result == STATE_WARNING)
						if (entry[line].descr)
							errmsg = entry[line].descr;
					if (code_mask[entry[line].code] == TRUE ) {
						result = STATE_CRITICAL;
						if (entry[line].descr)
							errmsg = entry[line].descr;
					}
					break;

				default:

					result = STATE_UNKNOWN;
					if (entry[line].descr)
						errmsg = entry[line].descr;
					break;
			}

		} else {
			result = max_state(result,STATE_OK);
			if (result == STATE_OK)
				if (entry[line].descr)
					errmsg = entry[line].descr;
		}
	}

	/* no entries in prtAlarmTable or no description (KYOCERA), query ptrConsoleDisplayBufferText */
	if (line == 0 || entry[0].descr == NULL) {

		sprintf (command_line, "%s -On -v 1 -c %s %s %s",
			PATH_TO_SNMPWALK, community, address, PTR_CONSOLE_DISPLAY_BUFFER_TEXT);
	        
		#if DEBUG
    		printf("command   : %s\n",command_line);
	        #endif
		
		child_process = spopen (command_line);
		
		while (fgets (input_buffer, MAX_INPUT_BUFFER - 1, child_process)) {

			if (input_buffer[strlen (input_buffer) - 1] == '\n')
				input_buffer[strlen (input_buffer) - 1] = 0;

            		#if DEBUG
	                printf("cmd output: %s\n",input_buffer);
    		        #endif

		        temp_buffer = strtok (input_buffer, "\"");
			temp_buffer = strtok (NULL, "\"");

			if (temp_buffer) {
				errmsg[strlen(errmsg)] = ' ';
			        errmsg = strcat(errmsg,temp_buffer);
			}	
			result = max_state(result,STATE_OK);
		}
	}

	/* UNKNOWN if output found on stderr */
	if (fgets (input_buffer, MAX_INPUT_BUFFER - 1, child_stderr)) {
	
		result = STATE_UNKNOWN;
		
		/* remove CRLF */
		if (input_buffer[strlen (input_buffer) - 1] == '\n')
			input_buffer[strlen (input_buffer) - 1] = 0;
		sprintf (errmsg, "%s", input_buffer );

	}
	
	/* close stderr */
        (void) fclose (child_stderr);

	/* close the pipe */
	if (spclose (child_process))
		result = max_state(result,STATE_UNKNOWN);

	switch (result) {
	
		case STATE_OK:
		        printf("OK - Printer ok (\"%s\")\n",errmsg);
			break;
		
		case STATE_WARNING:
			printf("WARNING - \"%s\"\n",errmsg);
			break;
		
		case STATE_CRITICAL:
			printf("CRITICAL - \"%s\"\n",errmsg);
			break;
		
		default:
			printf("UNKNOWN - \"%s\"\n",errmsg);
			break;
	
	}
	return result;
}


/* process command-line arguments */
int process_arguments (int argc, char **argv)
{
	int c;

	int option = 0;
	static struct option longopts[] = {
		{"hostname", required_argument, 0, 'H'},
		{"community", required_argument, 0, 'C'},
/*  		{"critical",       required_argument,0,'c'}, */
/*  		{"warning",        required_argument,0,'w'}, */
/*  		{"port",           required_argument,0,'P'}, */
		{"training-level", required_argument, 0, 'l'},
		{"version", no_argument, 0, 'V'},
		{"help", no_argument, 0, 'h'},
		{0, 0, 0, 0}
	};

	if (argc < 2)
		return ERROR;

	while (1) {
		c = getopt_long (argc, argv, "+hVH:C:l:", longopts, &option);

		if (c == -1 || c == EOF || c == 1)
			break;

		switch (c) {
		case 'H':									/* hostname */
			if (is_host (optarg)) {
				address = strscpy(address, optarg) ;
			}
			else {
				usage2 (_("Invalid hostname/address"), optarg);
			}
			break;
		case 'C':									/* community */
			community = strscpy (community, optarg);
			break;
		case 'l':									/* min training level */
			training_level = atoi(optarg);
			break;
		case 'V':									/* version */
			print_revision (progname, revision);
			exit (STATE_OK);
		case 'h':									/* help */
			print_help ();
			exit (STATE_OK);
		case '?':									/* help */
			usage2 (_("Unknown argument"), optarg);
		}
	}

	c = optind;
	if (address == NULL) {
		if (is_host (argv[c])) {
			address = argv[c++];
		}
		else {
			usage2 (_("Invalid hostname/address"), argv[c]);
		}
	}
	
	if (community == NULL) {
		if (argv[c] != NULL )
			community = argv[c];
		else
			community = strdup (DEFAULT_COMMUNITY);
	}

	return OK;
}


void
print_help (void)
{
	print_revision (progname, revision);

	printf ("Copyright (c) 2005 Christian Schoeniger <csc@fes-aes.de>\n");
	printf (COPYRIGHT, copyright, email);

	printf (_("\
This plugin tests the STATUS of an printer with printmib support.\n\
ucd-snmp must be installed on the computer running the plugin.\n\n"));

	print_usage ();

	printf (_(UT_HELP_VRSN));

	printf (_("\
 -C, --community=STRING\n\
    The SNMP community name (default=%s)\n"), DEFAULT_COMMUNITY);

	printf (_(UT_SUPPORT));
}



void
print_usage (void)
{
	printf ("Usage: %s -H host [-C community]\n", progname);
}

char* convert (char* str) { /* converts OCTET STRING to DisplayString */

    char *tmp = malloc(MAX_INPUT_BUFFER);
    unsigned int d[4];

    if (str[0] == '"')
	    str++;
    if (str[strlen(str)-1] == '"')
	    str[strlen(str)-1] = 0;
    
    #if DEBUG
    printf("\tconvert(\"%s\") ",str);
    #endif

    if (sscanf(str,"%d %d %d %d",&d[0],&d[1],&d[2],&d[3])) {

    	    do {
		    if (strtoul(str,NULL,16) != 10)
			    sprintf(tmp,"%s%c",tmp,(int)strtol(str,NULL,16));
		    else
			    sprintf(tmp,"%s%c",tmp,' ');

    	    } while ((str = (strstr(++str," "))));

	    sprintf(tmp,"%s%c",tmp,0);
	    
    	    str = tmp;

	}

    #if DEBUG	
    printf("returns \"%s\"\n",str);
    #endif

    return str;

}
